@@ -22,11 +22,15 @@ See https://developers.google.com/elections-data/reference/
 from __future__ import print_function
 
 import argparse
+import cProfile
 import hashlib
+import io
 import os
+import pstats
 import re
 
 from civics_cdf_validator import base
+from civics_cdf_validator import gpunit_rules
 from civics_cdf_validator import loggers
 from civics_cdf_validator import rules
 from civics_cdf_validator import version
@@ -123,6 +127,10 @@ def add_validate_parser_args(parser, parser_validate):
       "--required_languages",
       help="Languages required by the AllLanguages check.",
       required=False)
+  parser_validate.add_argument(
+      "--profile_report",
+      help="Run profiling and print the execution report.",
+      required=False)
 
 
 def add_validate_parser_input_file_args(parser, parser_validate):
@@ -266,24 +274,31 @@ def compute_max_found_severity(exceptions_wrapper):
     return 0
 
 
+def exec_profiling(func):
+  """This is a decorator to add profiling to the feed validation."""
+  def add_profiling_if_needed(args):
+    if args is None or not args.profile_report:
+      return func(args)
+    pr = cProfile.Profile(builtins=False)
+    pr.enable()
+    result = func(args)
+    pr.disable()
+    s = io.StringIO()
+    ps = pstats.Stats(pr, stream=s).strip_dirs().sort_stats("cumulative")
+    ps.print_stats("rules")
+    print(s.getvalue())
+    return result
+
+  return add_profiling_if_needed
+
+
+@exec_profiling
 def feed_validation(options):
   """Validate the input feed depending on the user parameters."""
   rule_options = {}
-  if options.g:
-    rule_options.setdefault("ElectoralDistrictOcdId", []).append(
-        base.RuleOption("check_github", False))
-    rule_options.setdefault("GpUnitOcdId", []).append(
-        base.RuleOption("check_github", False))
-  if options.c:
-    rule_options.setdefault("ElectoralDistrictOcdId", []).append(
-        base.RuleOption("country_code", options.c))
-    rule_options.setdefault("GpUnitOcdId", []).append(
-        base.RuleOption("country_code", options.c))
-  if options.ocdid_file:
-    rule_options.setdefault("ElectoralDistrictOcdId", []).append(
-        base.RuleOption("local_file", options.ocdid_file))
-    rule_options.setdefault("GpUnitOcdId", []).append(
-        base.RuleOption("local_file", options.ocdid_file))
+  gpunit_rules.GpUnitOcdIdValidator.init_ocd_id_list(options.c,
+                                                     options.ocdid_file,
+                                                     not options.g)
   if options.required_languages:
     rule_options.setdefault("AllLanguages", []).append(
         base.RuleOption("required_languages",
